module LuxOptimisersExt

using Lux: Lux
using LuxDeviceUtils: AbstractLuxDevice, gpu_device
using Optimisers: Optimisers
using Random: Random

"""
    TrainState(rng::Random.AbstractRNG, model::Lux.AbstractExplicitLayer,
        optimizer::Optimisers.AbstractRule;
        transform_variables::Union{Function, AbstractLuxDevice}=gpu_device())

Constructor for [`TrainState`](@ref).

## Arguments

  - `rng`: Random Number Generator.
  - `model`: `Lux` model.
  - `optimizer`: Optimizer from `Optimisers.jl`.
  - `transform_variables`: Function to transform the variables of the model. Typically used
    to transfer variables to GPU / CPU.

## Returns

[`TrainState`](@ref) object.
"""
function Lux.Experimental.TrainState(
        rng::Random.AbstractRNG, model::Lux.AbstractExplicitLayer,
        optimizer::Optimisers.AbstractRule;
        transform_variables::Union{Function, AbstractLuxDevice}=gpu_device())
    ps, st = Lux.setup(rng, model) .|> transform_variables
    st_opt = Optimisers.setup(optimizer, ps)
    return Lux.Experimental.TrainState(model, ps, st, st_opt, 0)
end

function Lux.Experimental.apply_gradients(ts::Lux.Experimental.TrainState, grads)
    optimizer_state, ps = Optimisers.update(ts.optimizer_state, ts.parameters, grads)
    return Lux.Experimental.TrainState(
        ts.model, ps, ts.states, optimizer_state, ts.step + 1)
end

end
